using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using OpenXLive;
using OpenXLive.Features;
using OpenXLive.Forms;
using Microsoft.Phone.Notification;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text;

namespace PushDemoXNA
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        #region XNA Game Method

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        XLiveFormManager manager;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Extend battery life under lock.
            InactiveSleepTime = TimeSpan.FromSeconds(1);

            graphics.IsFullScreen = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            if (XLiveGameManager.CurrentSession == null)
            {
                GameSession session = XLiveGameManager.CreateSession("enMQ3rEC5jsw8XygdFHuGpfk");
                session.CreateSessionCompleted += new AsyncEventHandler(session_CreateSessionCompleted);

                manager = new XLiveFormManager(this, session);

                session.Open();  

                Components.Add(manager);
            }

            base.Initialize();
        }

        void session_CreateSessionCompleted(object sender, AsyncEventArgs e)
        {
            AsyncProcessResult result = e.Result;

            if (result.ReturnValue)
            {
                // TODO: Error Handler
                CreateNotificationChannel();
            }
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            if (manager.IsRunning)
            {
                TouchCollection Touches = TouchPanel.GetState();
                foreach (TouchLocation location in Touches)
                {
                    if (location.State == TouchLocationState.Pressed)
                    {
                        XLiveFormFactory.Factory.ShowForm("Lobby");
                        //CreateNotificationChannel();
                    }
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }

        #endregion

        #region Push Notification

        // The name of our push channel.
        string TileChannelName = "OpenXLiveTileNotificationChannel";
        string ToastChannelName = "OpenXLiveToastNotificationChannel";
        string RawChannelName = "OpenXLiveRawNotificationChannel";

        private void RegisterNotificationUri(Uri uri)
        {
            // Send Channel Uri to OpenXLive Hosting Server
            if (OpenXLive.XLiveGameManager.CurrentSession != null
                &&
                OpenXLive.XLiveGameManager.CurrentSession.IsValid
                &&
                uri != null)
            {
                OpenXLive.XLiveGameManager.CurrentSession.RegistNotificationUriCompleted
                    += new OpenXLive.Features.AsyncEventHandler(CurrentSession_RegistNotificationUriCompleted);
                OpenXLive.XLiveGameManager.CurrentSession.RegisterNotificationUri(uri);
            }
        }

        void CurrentSession_RegistNotificationUriCompleted(object sender, OpenXLive.AsyncEventArgs e)
        {
            OpenXLive.Features.AsyncProcessResult result = e.Result;

            if (!result.ReturnValue)
            {
                Debug.WriteLine("Channel Uri has been send to OpenXLive Hosting Server");
            }
            else
            {
                Debug.WriteLine(result.ErrorMessage);
            }
        }

        private void CreateNotificationChannel()
        {
            CreateTileNotification();

            //CreateToastNotification();

            //CreateRawNotification();
        }

        #region Toast Notification

        private void CreateToastNotification()
        {
            // Try to find the push channel.
            HttpNotificationChannel pushChannel = HttpNotificationChannel.Find(ToastChannelName);

            // If the channel was not found, then create a new connection to the push service.
            if (pushChannel == null)
            {
                pushChannel = new HttpNotificationChannel(ToastChannelName);

                // Register for all the events before attempting to open the channel.
                pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
                pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);

                // Register for this notification only if you need to receive the notifications while your application is running.
                pushChannel.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(PushChannel_ShellToastNotificationReceived);

                pushChannel.Open();

                // Bind this new channel for Tile events.
                pushChannel.BindToShellToast();
            }
            else
            {
                // The channel was already open, so just register for all the events.
                pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
                pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);

                // Register for this notification only if you need to receive the notifications while your application is running.
                pushChannel.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(PushChannel_ShellToastNotificationReceived);
            }

            // Send Channel Uri to OpenXLive Hosting Server
            RegisterNotificationUri(pushChannel.ChannelUri);
        }

        void PushChannel_ShellToastNotificationReceived(object sender, NotificationEventArgs e)
        {
            StringBuilder message = new StringBuilder();
            string relativeUri = string.Empty;

            message.AppendFormat("Received Toast {0}:\n", DateTime.Now.ToShortTimeString());

            // Parse out the information that was part of the message.
            foreach (string key in e.Collection.Keys)
            {
                message.AppendFormat("{0}: {1}\n", key, e.Collection[key]);

                if (string.Compare(
                    key,
                    "wp:Param",
                    System.Globalization.CultureInfo.InvariantCulture,
                    System.Globalization.CompareOptions.IgnoreCase) == 0)
                {
                    relativeUri = e.Collection[key];
                }
            }

            // Display a dialog of all the fields in the toast.
            MessageBox.Show(message.ToString());
        }

        #endregion

        #region Raw Notification

private void CreateRawNotification()
{
    // Try to find the push channel.
    HttpNotificationChannel pushChannel = HttpNotificationChannel.Find(RawChannelName);

    // If the channel was not found, then create a new connection to the push service.
    if (pushChannel == null)
    {
        pushChannel = new HttpNotificationChannel(RawChannelName);

        // Register for all the events before attempting to open the channel.
        pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
        pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);
        pushChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(PushChannel_HttpNotificationReceived);

        pushChannel.Open();

    }
    else
    {
        // The channel was already open, so just register for all the events.
        pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
        pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);
        pushChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(PushChannel_HttpNotificationReceived);

        // Display the URI for testing purposes. Normally, the URI would be passed back to your web service at this point.
        System.Diagnostics.Debug.WriteLine(pushChannel.ChannelUri.ToString());
        MessageBox.Show(String.Format("Channel Uri is {0}",
            pushChannel.ChannelUri.ToString()));

    }

    // Send Channel Uri to OpenXLive Hosting Server
    RegisterNotificationUri(pushChannel.ChannelUri);
}

void PushChannel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
{
    string message;

    using (System.IO.StreamReader reader = new System.IO.StreamReader(e.Notification.Body))
    {
        message = reader.ReadToEnd();
    }


    MessageBox.Show(String.Format("Received Notification {0}:\n{1}",
            DateTime.Now.ToShortTimeString(), message)
            );
}

        #endregion

        #region Tile Notification

        private void CreateTileNotification()
        {
            if (XLiveGameManager.CurrentSession != null && XLiveGameManager.CurrentSession.IsValid)
            {
                // Try to find the push channel.
                HttpNotificationChannel pushChannel = HttpNotificationChannel.Find(TileChannelName);

                // If the channel was not found, then create a new connection to the push service.
                if (pushChannel == null)
                {
                    pushChannel = new HttpNotificationChannel(TileChannelName);

                    // Register for all the events before attempting to open the channel.
                    pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
                    pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);

                    pushChannel.Open();

                    // Bind this new channel for Tile events.
                    if (pushChannel != null && !pushChannel.IsShellTileBound)
                    {
                        var allowedDomains = new Collection<Uri> { new Uri("http://picture.openxlive.com/") };
                        pushChannel.BindToShellTile(allowedDomains);
                    }
                }
                else
                {
                    // The channel was already open, so just register for all the events.
                    pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
                    pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);
                }

                RegisterNotificationUri(pushChannel.ChannelUri);
            }
        }

        void PushChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            if (e.ChannelUri != null)
            {
                RegisterNotificationUri(e.ChannelUri);
            }

            // Display the new URI for testing purposes. Normally, the URI would be passed back to your web service at this point.
            System.Diagnostics.Debug.WriteLine(e.ChannelUri.ToString());
        }

        void PushChannel_ErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            // Error handling logic for your particular application would be here.
            MessageBox.Show(String.Format("A push notification {0} error occurred.  {1} ({2}) {3}",
                    e.ErrorType, e.Message, e.ErrorCode, e.ErrorAdditionalData));
        }

        #endregion


        #endregion
    }
}
